//import crypto from 'crypto'
import { globalPassword } from './pwgen';
let password: string;
let salt;
let ciphertext;
let iv;
globalPassword.subscribe(value => {
  password = value;
})

/*
Fetch the contents of the "message" textbox, and encode it
in a form we can use for the encrypt operation.
*/
function getStringEncoding(string: string) {
  let enc = new TextEncoder();
  return enc.encode(string);
}
/*
Get some key material to use as input to the deriveKey method.
The key material is a password supplied by the user.
*/
function getKeyMaterial() {
    let enc = new TextEncoder();
    return window.crypto.subtle.importKey(
      "raw", 
      enc.encode(password), 
      {name: "PBKDF2"}, 
      false, 
      ["deriveBits", "deriveKey"]
    );
}
/*
Given some key material and some random salt
derive an AES-GCM key using PBKDF2.
*/
function getKey(keyMaterial, salt) {
  return window.crypto.subtle.deriveKey(
    {
      "name": "PBKDF2",
      salt: salt, 
      "iterations": 100000,
      "hash": "SHA-256"
    },
    keyMaterial,
    { "name": "AES-GCM", "length": 256},
    true,
    [ "encrypt", "decrypt" ]
  );
}

/*
Derive a key from a password supplied by the user, and use the key
to encrypt the message.
Update the "ciphertextValue" box with a representation of part of
the ciphertext.
*/
export async function encrypt(string: string) {
  let dec = new TextDecoder();
  let keyMaterial = await getKeyMaterial();
  salt = window.crypto.getRandomValues(new Uint8Array(16));
  let key = await getKey(keyMaterial, salt);
  iv = window.crypto.getRandomValues(new Uint8Array(12));
  let encoded = getStringEncoding(string);

  ciphertext = await window.crypto.subtle.encrypt(
    {
      name: "AES-GCM",
      iv: iv
    },
    key,
    encoded
  );
  let decodedCiphertext = dec.decode(ciphertext);
  return decodedCiphertext

}

/*
Derive a key from a password supplied by the user, and use the key
to decrypt the ciphertext.
If the ciphertext was decrypted successfully,
update the "decryptedValue" box with the decrypted value.
If there was an error decrypting,
update the "decryptedValue" box with an error message.
*/
export async function decrypt(encryptedContent) {
  let keyMaterial = await getKeyMaterial();
  let key = await getKey(keyMaterial, salt);

  try {
    let decrypted = await window.crypto.subtle.decrypt(
      {
        name: "AES-GCM",
        iv: iv
      },
      key,
      encryptedContent
    );

    let dec = new TextDecoder();
    return dec.decode(decrypted);
  } catch (e) {
    console.log(e)
    return `Decryption error: ${e}`
  }
}
// Hacked together version of https://github.com/mdn/dom-examples/blob/main/web-crypto/derive-key/pbkdf2.jshttps://gist.github.com/ChaoLiangSuper/0e13f77712b68682f0d8ebabb2d63aa8